@@ -1,9 +1,6 @@
-package POE::Component::Client::HTTP::Request;
-BEGIN {
-  $POE::Component::Client::HTTP::Request::VERSION = '0.900_161';
-}
-# vim: ts=2 sw=2 expandtab
+# $Id: Request.pm 222 2005-09-17 18:04:07Z rcaputo $
 
+package POE::Component::Client::HTTP::Request;
 use strict;
 use warnings;
 
@@ -11,49 +8,37 @@ use POE;
 
 use Carp;
 use HTTP::Status;
-use Errno qw(ETIMEDOUT);
-
-BEGIN {
-  local $SIG{'__DIE__'} = 'DEFAULT';
-  # Allow more finely grained timeouts if Time::HiRes is available.
-  # This code is also in POE::Component::Client::HTTP
-  eval {
-    require Time::HiRes;
-    Time::HiRes->import("time");
-  };
-}
 
 # Unique request ID, independent of wheel and timer IDs.
 my $request_seq = 0;
 
-use constant DEBUG => 0;
+sub DEBUG () { 0 }
 
-use constant REQ_ID            =>  0;
-use constant REQ_POSTBACK      =>  1;
-use constant REQ_CONNECTION    =>  2;
-use constant REQ_HTTP_REQUEST  =>  3;
-use constant REQ_STATE         =>  4;
-use constant REQ_RESPONSE      =>  5;
-use constant REQ_BUFFER        =>  6;
-use constant REQ_OCTETS_GOT    =>  8;
-use constant REQ_TIMER         =>  9;
-use constant REQ_PROG_POSTBACK => 10;
-use constant REQ_USING_PROXY   => 11;
-use constant REQ_HOST          => 12;
-use constant REQ_PORT          => 13;
-use constant REQ_HISTORY       => 14;
-use constant REQ_START_TIME    => 15;
-use constant REQ_FACTORY       => 16;
-use constant REQ_CONN_ID       => 17;
-use constant REQ_PEERNAME      => 18;
+sub REQ_ID            () {  0 }
+sub REQ_POSTBACK      () {  1 }
+sub REQ_CONNECTION    () {  2 }
+sub REQ_REQUEST       () {  3 }
+sub REQ_STATE         () {  4 }
+sub REQ_RESPONSE      () {  5 }
+sub REQ_BUFFER        () {  6 }
 
-use constant RS_CONNECT        => 0x01; # establishing a connection
-use constant RS_SENDING        => 0x02; # sending request to server
-use constant RS_IN_HEAD        => 0x04; # waiting for or receiving headers
-use constant RS_REDIRECTED     => 0x08; # request has been redirected
-use constant RS_IN_CONTENT     => 0x20; # waiting for or receiving content
-use constant RS_DONE           => 0x40; # received full content
-use constant RS_POSTED         => 0x80; # we have posted back a response
+sub REQ_OCTETS_GOT    () {  8 }
+sub REQ_TIMER         () {  9 }
+sub REQ_PROG_POSTBACK () { 10 }
+sub REQ_USING_PROXY   () { 11 }
+sub REQ_HOST          () { 12 }
+sub REQ_PORT          () { 13 }
+sub REQ_HISTORY       () { 14 }
+sub REQ_START_TIME    () { 15 }
+sub REQ_FACTORY       () { 16 }
+
+sub RS_CONNECT      () { 0x01 }
+sub RS_SENDING      () { 0x02 }
+sub RS_IN_HEAD      () { 0x04 }
+sub RS_REDIRECTED   () { 0x08 }
+sub RS_IN_CONTENT   () { 0x20 }
+sub RS_DONE         () { 0x40 }
+sub RS_POSTED       () { 0x80 }
 
 sub import {
   my ($class) = shift;
@@ -64,10 +49,10 @@ sub import {
     if ($tag eq ':fields') {
       foreach my $sub (
         qw(
-          REQ_ID REQ_POSTBACK REQ_CONNECTION REQ_HTTP_REQUEST REQ_STATE
+          REQ_ID REQ_POSTBACK REQ_CONNECTION REQ_REQUEST REQ_STATE
           REQ_RESPONSE REQ_BUFFER REQ_OCTETS_GOT REQ_TIMER
           REQ_PROG_POSTBACK REQ_USING_PROXY REQ_HOST REQ_PORT
-          REQ_HISTORY REQ_START_TIME REQ_CONN_ID REQ_PEERNAME
+          REQ_HISTORY REQ_START_TIME
         )
       ) {
         no strict 'refs';
@@ -104,10 +89,12 @@ sub new {
   croak "Request must be a HTTP::Request object"
     unless (UNIVERSAL::isa ($params{'Request'}, "HTTP::Request"));
 
+  #croak "need a Tag parameter" unless (defined $params{'Tag'});
+
   croak "need a Factory parameter" unless (defined $params{'Factory'});
 
-  my ($http_request, $postback, $progress, $factory) =
-    @params{qw(Request Postback Progress Factory)};
+  my ($http_request, $postback, $tag, $progress, $factory) =
+    @params{qw(Request Postback Tag Progress Factory)};
 
   my $request_id = ++$request_seq;
   DEBUG and warn "REQ: creating a request ($request_id)";
@@ -147,7 +134,7 @@ sub new {
     $request_id,        # REQ_ID
     $postback,          # REQ_POSTBACK
     undef,              # REQ_CONNECTION
-    $http_request,      # REQ_HTTP_REQUEST
+    $http_request,      # REQ_REQUEST
     RS_CONNECT,         # REQ_STATE
     undef,              # REQ_RESPONSE
     '',                 # REQ_BUFFER
@@ -161,10 +148,8 @@ sub new {
     undef,              # REQ_HISTORY
     time(),             # REQ_START_TIME
     $factory,           # REQ_FACTORY
-    undef,              # REQ_CONN_ID
-    undef,              # REQ_PEERNAME
-  ];
-  return bless $self, $class;
+   ];
+   return bless $self, $class;
 }
 
 sub return_response {
@@ -210,21 +195,14 @@ sub add_eof {
 
   unless (defined $self->[REQ_RESPONSE]) {
     # XXX I don't know if this is actually used
-    $self->error(400, "incomplete response a " . $self->[REQ_ID]);
+    $self->error(400, "incomplete response " . $self->[REQ_ID]);
     return;
   }
 
   # RFC 2616: "If a message is received with both a Transfer-Encoding
   # header field and a Content-Length header field, the latter MUST be
   # ignored."
-  #
-  # Google returns a Content-Length header with its HEAD request,
-  # generating "incomplete response" errors.  Added a special case to
-  # ignore content for HEAD requests.  This may thwart keep-alive,
-  # however.
-
   if (
-    $self->[REQ_HTTP_REQUEST]->method() ne "HEAD" and
     defined $self->[REQ_RESPONSE]->content_length and
     not defined $self->[REQ_RESPONSE]->header("Transfer-Encoding") and
     $self->[REQ_OCTETS_GOT] < $self->[REQ_RESPONSE]->content_length
@@ -233,13 +211,7 @@ sub add_eof {
       "got " . $self->[REQ_OCTETS_GOT] . " of " .
       $self->[REQ_RESPONSE]->content_length
     );
-
-    $self->error(
-      400,
-      "incomplete response b " . $self->[REQ_ID] . ".  Wanted " .
-      $self->[REQ_RESPONSE]->content_length() . " octets.  Got " .
-      $self->[REQ_OCTETS_GOT] . "."
-    );
+    $self->error(400, "incomplete response " . $self->[REQ_ID]);
   }
   else {
     $self->[REQ_STATE] |= RS_DONE;
@@ -265,29 +237,23 @@ sub add_content {
   # therein.  If it's done the former, then we're safe.  Otherwise
   # we also need to C<use bytes>.
   # TODO: write test(s) for this.
-
   my $this_chunk_length = length($self->[REQ_BUFFER]);
   $self->[REQ_OCTETS_GOT] += $this_chunk_length;
 
-  my $max = $self->[REQ_FACTORY]->max_response_size();
+  my $max = $self->[REQ_FACTORY]->max_response_size;
 
   DEBUG and warn(
     "REQ: request ", $self->ID,
     " received $self->[REQ_OCTETS_GOT] bytes; maximum is $max"
   );
 
-  # Fail if we've gone over the maximum content size to return.
   if (defined $max and $self->[REQ_OCTETS_GOT] > $max) {
-    $self->error(
-      406,
-      "Response content is longer than specified MaxSize of $max.  " .
-      "Use range requests to retrieve specific amounts of content."
-    );
-
-    $self->[REQ_STATE] |= RS_DONE;
-    $self->[REQ_STATE] &= ~RS_IN_CONTENT;
-    $self->[REQ_CONNECTION]->close();
-    return 1;
+    # We've gone over the maximum content size to return.  Chop it # back.
+    my $over = $self->[REQ_OCTETS_GOT] - $max;
+    $self->[REQ_OCTETS_GOT] -= $over;
+    substr($self->[REQ_BUFFER], -$over) = "";
+    #TODO: ??
+    #$self->[REQ_STATE] |= RS_DONE;
   }
 
   # keep this for the progress callback (it gets cleared in return_response
@@ -312,6 +278,15 @@ sub add_content {
     );
   };
 
+  if (defined $max and $self->[REQ_OCTETS_GOT] >= $max) {
+    DEBUG and warn(
+      "REQ: request ", $self->ID, " has a full response... moving to done."
+    );
+    $self->[REQ_STATE] |= RS_DONE;
+    $self->[REQ_STATE] &= ~RS_IN_CONTENT;
+    return 1;
+  }
+
   if ($self->[REQ_RESPONSE]->content_length) {
 
     # Report back progress
@@ -333,7 +308,6 @@ sub add_content {
       return 1;
     }
   }
-
   return 0;
 }
 
@@ -430,20 +404,18 @@ sub check_redirect {
 
   return undef unless ($self->[REQ_RESPONSE]->is_redirect);
 
-  # Make sure to frob any cookies set.  Redirect cookies are cookies, too!
-  $self->[REQ_FACTORY]->frob_cookies($self->[REQ_RESPONSE]);
-
-  my $location_uri = $self->[REQ_RESPONSE]->header('Location');
-
-  DEBUG and warn "REQ: Preparing redirect to $location_uri";
+  my $new_uri = $self->[REQ_RESPONSE]->header ('Location');
+  DEBUG and warn "REQ: Preparing redirect to $new_uri";
   my $base = $self->[REQ_RESPONSE]->base();
-  $location_uri = URI->new($location_uri, $base)->abs($base);
-  DEBUG and warn "RED: Actual redirect uri is $location_uri";
+  $new_uri = URI->new($new_uri, $base)->abs($base);
+  DEBUG and warn "RED: Actual redirect uri is $new_uri";
 
   my $prev = $self;
   my $history = 0;
   while ($prev = $prev->[REQ_HISTORY]) {
-    last if ++$history > $max;
+    $history++;
+    $history = $max + 1 if ($prev->[REQ_REQUEST]->uri eq $new_uri);
+    last if ($history > $max);
   }
 
   if ($history >= $max) {
@@ -451,25 +423,10 @@ sub check_redirect {
     DEBUG and warn "RED: Too much redirection";
   }
   else { # All fine, yield new request and mark this disabled.
-    my $newrequest = $self->[REQ_HTTP_REQUEST]->clone();
-
-    # Sanitize new request per rt #30400.
-    # TODO - What other headers are security risks?
-    $newrequest->remove_header('Cookie');
-
+    my $newrequest = $self->[REQ_REQUEST]->clone();
     DEBUG and warn "RED: new request $newrequest";
-    $newrequest->uri($location_uri);
-
-    # Don't change the Host header on a relative redirect.  This
-    # allows the HTTP::Request's Host to remain intact, per
-    # rt.cpan.org #63990.
-    if (defined $location_uri->scheme()) {
-      DEBUG and warn "RED: redirecting to absolute location $location_uri";
-      _set_host_header($newrequest);
-    }
-    else {
-      DEBUG and warn "RED: no new Host for relative redirect to $location_uri";
-    }
+    $newrequest->uri($new_uri);
+    _set_host_header ($newrequest);
 
     $self->[REQ_STATE] = RS_REDIRECTED;
     DEBUG and warn "RED: new request $newrequest";
@@ -483,7 +440,7 @@ sub send_to_wheel {
 
   $self->[REQ_STATE] = RS_SENDING;
 
-  my $http_request = $self->[REQ_HTTP_REQUEST];
+  my $http_request = $self->[REQ_REQUEST];
 
   # MEXNIX 2002-06-01: Check for proxy.  Request query is a bit
   # different...
@@ -500,12 +457,9 @@ sub send_to_wheel {
     $http_request->method() . ' ' .
     $request_uri . ' ' .
     $http_request->protocol() . "\x0D\x0A" .
-    $http_request->headers_as_string("\x0D\x0A") . "\x0D\x0A"
+    $http_request->headers_as_string("\x0D\x0A") . "\x0D\x0A" .
+    $http_request->content() # . "\x0D\x0A"
   );
- 
-  if ( !ref $http_request->content() ) {
-    $request_string .= $http_request->content(); # . "\x0D\x0A"
-  }
 
   DEBUG and do {
     my $formatted_request_string = $request_string;
@@ -522,17 +476,12 @@ sub send_to_wheel {
 sub wheel {
   my ($self) = @_;
 
-  # FIXME - We don't support older versions of POE.  Remove this chunk
-  # of code when we're not fixing something else.
-  #
   #if (defined $new_wheel) {
-  #   Switch wheels.  This is cumbersome, but it works around a bug in
-  #   older versions of POE.
-  #  $self->[REQ_WHEEL] = undef;
-  #  $self->[REQ_WHEEL] = $new_wheel;
+    # Switch wheels.  This is cumbersome, but it works around a bug in
+    # older versions of POE.
+    #$self->[REQ_WHEEL] = undef;
+    #$self->[REQ_WHEEL] = $new_wheel;
   #}
-
-  return unless $self->[REQ_CONNECTION];
   return $self->[REQ_CONNECTION]->wheel;
 }
 
@@ -541,41 +490,30 @@ sub error {
 
   my $nl = "\n";
 
-  my $http_msg = status_message($code);
-  my $r = HTTP::Response->new($code, $http_msg, [ 'X-PCCH-Errmsg', $message ]);
+  my $r = HTTP::Response->new($code);
+  my $http_msg = status_message ($code);
   my $m = (
     "<html>$nl"
     . "<HEAD><TITLE>Error: $http_msg</TITLE></HEAD>$nl"
     . "<BODY>$nl"
     . "<H1>Error: $http_msg</H1>$nl"
     . "$message$nl"
-    . "<small>This is a client error, not a server error.</small>$nl"
     . "</BODY>$nl"
     . "</HTML>$nl"
   );
 
-  $r->content($m);
-  $r->request($self->[REQ_HTTP_REQUEST]);
+  $r->content ($m);
   $self->[REQ_POSTBACK]->($r);
-  $self->[REQ_STATE] |= RS_POSTED;
 }
 
 sub connect_error {
-  my ($self, $operation, $errnum, $errstr) = @_;
+  my ($self, $message) = @_;
 
   my $host = $self->[REQ_HOST];
   my $port = $self->[REQ_PORT];
 
-  if ($operation eq "connect" and $errnum == ETIMEDOUT) {
-    $self->error(408, "Connection to $host:$port failed: timeout");
-  }
-  else {
-    $self->error(
-      RC_INTERNAL_SERVER_ERROR,
-      "Connection to $host:$port failed: $operation error $errnum: $errstr"
-    );
-  }
-
+  $message = "Cannon connect to $host:$port ($message)";
+  $self->error (RC_INTERNAL_SERVER_ERROR, $message);
   return;
 }
 
@@ -586,7 +524,7 @@ sub port { shift->[REQ_PORT] }
 sub scheme {
   my $self = shift;
 
-  $self->[REQ_USING_PROXY] ? 'http' : $self->[REQ_HTTP_REQUEST]->uri->scheme;
+  $self->[REQ_USING_PROXY] ? 'http' : $self->[REQ_REQUEST]->uri->scheme;
 }
 
 sub DESTROY {
@@ -597,189 +535,3 @@ sub DESTROY {
 }
 
 1;
-
-__END__
-
-=head1 NAME
-
-POE::Component::Client::HTTP::Request - an HTTP request class
-
-=head1 VERSION
-
-version 0.900_161
-
-=head1 SYNOPSIS
-
-  # Used internally by POE::Component::Client::HTTP
-
-=head1 DESCRIPTION
-
-POE::Component::Client::HTTP::Request encapsulates the state of
-requests POE::Component::Client::HTTP requests throughout their life
-cycles.  There turns out to be a lot of state to manage.
-
-=head1 CONSTRUCTOR
-
-=head2 new NAMED_PARAMETERS
-
-Create a POE::Component::Client::HTTP object to manage a request.  The
-constructor takes several named parameters:
-
-=over 2
-
-=item Request => HTTP_REQUEST
-
-A POE::Component::Client::HTTP::Request object encapsulates a plain
-HTTP::Request.  Required.
-
-=item Factory => POE_COMPONENT_CLIENT_HTTP_REQUESTFACTORY
-
-The request may create additional requests during its lifetime, for
-example when following redirects.  The Factory parameter specifies the
-POE::Component::Client::HTTP::RequestFactory that may be used to
-create them.  Required.
-
-=item Postback => RESPONSE_POSTBACK
-
-POE::Component::Client::HTTP creates a postback that will be used to
-send responses to the requesting session.  Required.
-
-=item Progress => PROGRESS_POSTBACK
-
-Sets the progress notification if the user has requested progress
-events.  Optional.
-
-=item Proxy
-
-Sets the proxy used for this request, if requested by the user.
-Optional.
-
-=back
-
-=head1 METHODS
-
-=head2 ID
-
-Return the request's unique ID.
-
-=head2 return_response
-
-Sends a response back to the user's session.  Called by
-POE::Component::Client::HTTP when a complete response has arrived.
-
-=head2 add_eof
-
-Called by POE::Component::Client::HTTP to indicate EOF has arrived.
-
-=head2 add_content PARSED_DATA
-
-Called by POE::Component::Client::HTTP to add content data to an
-incrementally built response.  If PARSED_DATA is an object, it is
-treated like an HTTP::Headers object and its headers are assimilated
-into the response being built by the request.  Otherwise the
-PARSED_DATA is appended to the response's content.
-
-=head2 timer TIMER
-
-Accessor to manipulate the request's timeout timer.  Sets the
-request's timer if TIMER is specified, otherwise merely fetches the
-one currently associated with the request.
-
-=head2 create_timer TIMEOUT
-
-Creates and sets a timer for this request.  TIMEOUT is the number of
-seconds this request may live.
-
-=head2 remove_timeout
-
-Turn off the timer associated with this request, and discard it.
-
-=head2 postback POSTBACK
-
-Accessor to manipulate the postback associated with this request.
-Sets the postback if POSTBACK is defined, otherwise merely fetches it.
-
-=head2 does_redirect SOMETHING
-
-FIXME - Not sure what this accessor does.
-
-=head2 check_redirect
-
-Check whether the last response is a redirect, the request is
-permitted to follow redirects, and the maximum number of redirects has
-not been met.  Initiate a redirect if all conditions are favorable.
-
-=head2 send_to_wheel
-
-Transmit the request to the socket associated with this request.
-
-=head2 wheel
-
-An accessor to return the wheel associated with this request.
-
-=head2 error ERROR_CODE, ERROR_MESSAGE
-
-Generate an error response, and post it back to the user's session.
-
-=head2 connect_error CONNECT_FAILURE_MESSAGE
-
-Generate a connection error response, and post it back to the user's
-session.
-
-=head2 host
-
-Return the host this request is attempting to work with.
-
-=head2 port
-
-Return the port this request is attempting to work with.
-
-=head2 scheme
-
-Return the scheme for this request.
-
-=head1 SEE ALSO
-
-L<POE::Component::Client::HTTP>
-L<POE>
-
-=head1 BUGS
-
-None are currently known.
-
-=head1 AUTHOR & COPYRIGHTS
-
-POE::Component::Client::HTTP::Request is
-
-=over 2
-
-=item
-
-Copyright 2004-2005 Martijn van Beers
-
-=item
-
-Copyright 2006 Rocco Caputo
-
-=back
-
-All rights are reserved.  POE::Component::Client::HTTP::Request is
-free software; you may redistribute it and/or modify it under the same
-terms as Perl itself.
-
-=head1 CONTRIBUTORS
-
-Your name could be here.
-
-=head1 CONTACT
-
-Rocco may be contacted by e-mail via L<mailto:rcaputo@cpan.org>, and
-Martijn may be contacted by email via L<mailto:martijn@cpan.org>.
-
-The preferred way to report bugs or requests is through RT though.
-See L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=POE-Component-Client-HTTP>
-or mail L<mailto:bug-POE-Component-Client-HTTP@rt.cpan.org>
-
-For questions, try the L<POE> mailing list (poe@perl.org)
-
-=cut
\ No newline at end of file
